@@ -486,7 +486,9 @@ iERR _ion_reader_binary_get_type(ION_READER *preader, ION_TYPE *p_value_type)
         // the next value. All other values must be positioned before the contents of the current value.
         if (binary->_value_type != tid_SYMBOL)
         {
-            FAILWITH(IERR_INVALID_STATE);
+            // This matches the behavior of the text reader
+            *p_value_type = tid_none;
+            SUCCEED();
         }
     }
 
@@ -1004,7 +1006,17 @@ iERR _ion_reader_binary_read_int64(ION_READER *preader, int64_t *p_value)
     IONCHECK(ion_binary_read_uint_64(preader->istream, len, &unsignedInt64));
 
     is_negative = (tid == TID_NEG_INT)? TRUE: FALSE;
-    IONCHECK(cast_to_int64(unsignedInt64, is_negative, p_value));
+
+    {
+        iERR cast_result = cast_to_int64(unsignedInt64, is_negative, p_value);
+
+        // if an error is thrown reset reader stream's current pointer to the beginning of current value again
+        if (cast_result != IERR_OK) {
+            preader->istream->_curr -= len;
+            FAILWITH(cast_result);
+        }
+    }
+
     if (is_negative && *p_value == 0) {
         FAILWITH(IERR_INVALID_BINARY);
     }
