@@ -1,22 +1,21 @@
-use std::cell::RefCell;
 use std::cmp::min;
 use std::fmt::{Display, Write};
 use std::fs::File;
 use std::io;
 use std::io::BufWriter;
 use std::ops::Range;
-use std::rc::Rc;
 use std::str::{from_utf8_unchecked, FromStr};
 
 use anyhow::{bail, Context, Result};
 use clap::{App, Arg, ArgMatches};
 use colored::Colorize;
-use ion_rs::{BinaryIonCursor, IonType, Reader, SymbolTable, SystemEventHandler};
 use ion_rs::result::IonResult;
 use ion_rs::text::writer::TextWriter;
+use ion_rs::{IonType, RawBinaryReader, SystemReader, SystemStreamItem};
 use memmap::MmapOptions;
 
-const ABOUT: &str = "Displays hex-encoded binary Ion alongside its equivalent text for human-friendly debugging.";
+const ABOUT: &str =
+    "Displays hex-encoded binary Ion alongside its equivalent text for human-friendly debugging.";
 
 // Creates a `clap` (Command Line Arguments Parser) configuration for the `inspect` command.
 // This function is invoked by the `inspect` command's parent, `beta`, so it can describe its
@@ -56,8 +55,8 @@ Ion version markers and symbol tables in the bytes being skipped will
 still be displayed. If the requested number of bytes falls in the
 middle of a value, the whole value (complete with field ID and
 annotations if applicable) will be displayed. If the value is nested
-in one or more containers, those containers will be displayed too."
-                )
+in one or more containers, those containers will be displayed too.",
+                ),
         )
         .arg(
             // This is named `limit-bytes` instead of `limit` to accommodate a future `limit-values` option.
@@ -70,29 +69,22 @@ in one or more containers, those containers will be displayed too."
                 .long_help(
                     "When specified, the inspector will stop printing values after
 processing `n` bytes of Ion data. If `n` falls within a value, the
-complete value will be displayed."
-                )
+complete value will be displayed.",
+                ),
         )
 }
 
-// Create a type alias to simplify working with a shared, mutable reference to our output stream.
-type OutputRef = Rc<RefCell<dyn io::Write>>;
+// Create a type alias to simplify working with a shared reference to our output stream.
+type OutputRef = Box<dyn io::Write>;
 // * The output stream could be STDOUT or a file handle, so we use `dyn io::Write` to abstract
 //   over the two implementations.
-// * The output stream will be shared by the IonInspector and the SystemEventHandler, so we use
-//   an reference counting pointer (`Rc`) to allow each of them to own a reference to it.
-// * Each entity that holds a reference to the output stream will need to mutate it, so we wrap it
-//   in a `RefCell`, which adds a small amount of runtime cost to guarantee that only one owner
-//   attempts to modify it at a time.
 // * The Drop implementation will ensure that the output stream is flushed when the last reference
-//   is dropped, so we don't need to do this manually.
+//   is dropped, so we don't need to do that manually.
 
 // This function is invoked by the `inspect` command's parent, `beta`.
 pub fn run(_command_name: &str, matches: &ArgMatches<'static>) -> Result<()> {
     // --skip-bytes has a default value, so we can unwrap this safely.
-    let skip_bytes_arg = matches
-        .value_of("skip-bytes")
-        .unwrap();
+    let skip_bytes_arg = matches.value_of("skip-bytes").unwrap();
 
     let bytes_to_skip = usize::from_str(skip_bytes_arg)
         // The `anyhow` crate allows us to augment a given Result with some arbitrary context that
@@ -100,9 +92,7 @@ pub fn run(_command_name: &str, matches: &ArgMatches<'static>) -> Result<()> {
         .with_context(|| format!("Invalid value for '--skip-bytes': '{}'", skip_bytes_arg))?;
 
     // --limit-bytes has a default value, so we can unwrap this safely.
-    let limit_bytes_arg = matches
-        .value_of("limit-bytes")
-        .unwrap();
+    let limit_bytes_arg = matches.value_of("limit-bytes").unwrap();
 
     let mut limit_bytes = usize::from_str(limit_bytes_arg)
         .with_context(|| format!("Invalid value for '--limit-bytes': '{}'", limit_bytes_arg))?;
@@ -113,13 +103,13 @@ pub fn run(_command_name: &str, matches: &ArgMatches<'static>) -> Result<()> {
         limit_bytes = usize::MAX
     }
 
-    let output: OutputRef;
+    let mut output: OutputRef;
     // If the user has specified an output file, use it.
     if let Some(file_name) = matches.value_of("output") {
-        let output_file = File::create(file_name)
-            .with_context(|| format!("Could not open '{}'", file_name))?;
+        let output_file =
+            File::create(file_name).with_context(|| format!("Could not open '{}'", file_name))?;
         let buf_writer = BufWriter::new(output_file);
-        output = Rc::new(RefCell::new(buf_writer));
+        output = Box::new(buf_writer);
     } else {
         // Otherwise, write to STDOUT.
         // TODO: Using io::stdout() isn't ideal as each write to stdout requires acquiring the
@@ -127,15 +117,21 @@ pub fn run(_command_name: &str, matches: &ArgMatches<'static>) -> Result<()> {
         //       could use to avoid that. `io::stdout().lock()` won't work because io::stdout()
         //       (to which it refers) has a limited lifetime.
         let buf_writer = BufWriter::new(io::stdout());
-        output = Rc::new(RefCell::new(buf_writer));
+        output = Box::new(buf_writer);
     }
 
     // Run the inspector on each input file that was specified.
     if let Some(input_file_iter) = matches.values_of("input") {
         for input_file_name in input_file_iter {
-            let mut input_file = File::open(input_file_name)
+            let input_file = File::open(input_file_name)
                 .with_context(|| format!("Could not open '{}'", input_file_name))?;
-            inspect_file(input_file_name, &mut input_file, &output, bytes_to_skip, limit_bytes)?;
+            inspect_file(
+                input_file_name,
+                input_file,
+                &mut output,
+                bytes_to_skip,
+                limit_bytes,
+            )?;
         }
     } else {
         // If no input file was specified, run the inspector on STDIN.
@@ -145,36 +141,47 @@ pub fn run(_command_name: &str, matches: &ArgMatches<'static>) -> Result<()> {
         // a temporary file and then read from that.
 
         // Create a temporary file that will delete itself when the program ends.
-        let mut input_file = tempfile::tempfile()
-            .with_context(|| concat!(
+        let mut input_file = tempfile::tempfile().with_context(|| {
+            concat!(
                 "Failed to create a temporary file to store STDIN.",
                 "Try passing an --input flag instead."
-            ))?;
+            )
+        })?;
 
         // Pipe the data from STDIN to the temporary file.
         let mut writer = BufWriter::new(input_file);
         io::copy(&mut io::stdin(), &mut writer)
             .with_context(|| "Failed to copy STDIN to a temp file.")?;
         // Get our file handle back from the BufWriter
-        input_file = writer.into_inner()
+        input_file = writer
+            .into_inner()
             .with_context(|| "Failed to read from temp file containing STDIN data.")?;
         // Read from the now-populated temporary file.
-        inspect_file("STDIN temp file", &mut input_file, &output, bytes_to_skip, limit_bytes)?;
+        inspect_file(
+            "STDIN temp file",
+            input_file,
+            &mut output,
+            bytes_to_skip,
+            limit_bytes,
+        )?;
     }
     Ok(())
 }
 
 // Given a file, try to mmap() it and run the inspector over the resulting byte array.
-fn inspect_file(input_file_name: &str,
-                input_file: &mut File,
-                output: &OutputRef,
-                bytes_to_skip: usize,
-                limit_bytes: usize) -> Result<()> {
+fn inspect_file(
+    input_file_name: &str,
+    input_file: File,
+    output: &mut OutputRef,
+    bytes_to_skip: usize,
+    limit_bytes: usize,
+) -> Result<()> {
     // mmap involves operating system interactions that inherently place its usage outside of Rust's
     // safety guarantees. If the file is unexpectedly truncated while it's being read, for example,
     // problems could arise.
     let mmap = unsafe {
-        MmapOptions::new().map(&input_file)
+        MmapOptions::new()
+            .map(&input_file)
             .with_context(|| format!("Could not mmap '{}'", input_file_name))?
     };
 
@@ -184,106 +191,32 @@ fn inspect_file(input_file_name: &str,
     match ion_data {
         // Pattern match the byte array to verify it starts with an IVM
         [0xE0, 0x01, 0x00, 0xEA, ..] => {
-            let mut inspector = IonInspector::new(
-                ion_data,
-                Rc::clone(output),
-                bytes_to_skip,
-                limit_bytes,
-            );
-
-            write_header(&output)?;
+            write_header(output)?;
+            let mut inspector = IonInspector::new(ion_data, output, bytes_to_skip, limit_bytes);
             // This inspects all values at the top level, recursing as necessary.
             inspector.inspect_level()?;
         }
         _ => {
             // bail! constructs an `anyhow::Result` with the given context and returns.
-            bail!("Input file '{}' does not appear to be binary Ion.", input_file_name);
+            bail!(
+                "Input file '{}' does not appear to be binary Ion.",
+                input_file_name
+            );
         }
     };
     Ok(())
 }
 
-// The ion::Reader type allows you to specify an event handler to react to low-level events in the
-// stream being read. This type summarizes them; it doesn't write out their full hex encoding,
-// it just writes a comment describing the event in the text Ion column.
-struct SystemLevelEventSummarizer {
-    output: OutputRef,
-    text_buffer: String,
-}
-
-impl SystemLevelEventSummarizer {
-    pub fn new(output: OutputRef) -> SystemLevelEventSummarizer {
-        SystemLevelEventSummarizer {
-            output,
-            text_buffer: String::with_capacity(512),
-        }
-    }
-}
-
 const IVM_HEX: &str = "e0 01 00 ea";
 const IVM_TEXT: &str = "// Ion 1.0 Version Marker";
 // System events (IVM, symtabs) are always at the top level.
 const SYSTEM_EVENT_INDENTATION: &str = "";
-
-impl SystemEventHandler for SystemLevelEventSummarizer {
-    // TODO: At the moment, the SystemEventHandler trait's functions do not have a return type that
-    //       would allow errors to bubble up. If writing to the output stream fails for some
-    //       reason, the program will end and a more terse error message will be displayed.
-    //       See: https://github.com/amzn/ion-rust/issues/118
-    fn on_ivm(&mut self, _ion_version: (u8, u8)) {
-        output(
-            &self.output,
-            None,
-            None,
-            SYSTEM_EVENT_INDENTATION,
-            IVM_HEX,
-            IVM_TEXT.dimmed(),
-        ).expect("output() failure from on_ivm()");
-    }
-
-    fn on_symbol_table_append(&mut self, symbol_table: &SymbolTable, starting_id: usize) {
-        self.text_buffer.clear();
-        self.text_buffer.push_str("// Local symbol table append: [\"");
-        join_into(&mut self.text_buffer, "\", \"", symbol_table.symbols_tail(starting_id).iter());
-        self.text_buffer.push_str("\"]");
-        output(
-            &self.output,
-            None,
-            None,
-            SYSTEM_EVENT_INDENTATION,
-            "...",
-            &self.text_buffer.dimmed(),
-        ).expect("output() failure from on_symbol_table_append()");
-    }
-
-    fn on_symbol_table_reset(&mut self, symbol_table: &SymbolTable) {
-        const ION_1_0_SYSTEM_TABLE_LENGTH: usize = 10;
-        self.text_buffer.clear();
-        if symbol_table.len() > ION_1_0_SYSTEM_TABLE_LENGTH {
-            self.text_buffer.push_str("// New local symbol table: [\"");
-            join_into(&mut self.text_buffer, "\", \"", symbol_table.symbols_tail(ION_1_0_SYSTEM_TABLE_LENGTH).iter());
-            self.text_buffer.push_str("\"]");
-        } else {
-            self.text_buffer.push_str("// Using system symbol table");
-        }
-
-        output(
-            &self.output,
-            None,
-            None,
-            SYSTEM_EVENT_INDENTATION,
-            "...",
-            &self.text_buffer.dimmed(),
-        ).expect("output() failure from on_symbol_table_reset()");
-    }
-}
-
 const LEVEL_INDENTATION: &str = "  "; // 2 spaces per level
 const TEXT_WRITER_INITIAL_BUFFER_SIZE: usize = 128;
 
-struct IonInspector<'input> {
-    output: OutputRef,
-    reader: Reader<BinaryIonCursor<io::Cursor<&'input [u8]>>>,
+struct IonInspector<'a> {
+    output: &'a mut OutputRef,
+    reader: SystemReader<RawBinaryReader<io::Cursor<&'a [u8]>>>,
     bytes_to_skip: usize,
     limit_bytes: usize,
     // Reusable buffer for formatting bytes as hex
@@ -298,10 +231,14 @@ struct IonInspector<'input> {
     text_ion_writer: TextWriter<Vec<u8>>,
 }
 
-impl<'input> IonInspector<'input> {
-    fn new(input: &'input [u8], out: OutputRef, bytes_to_skip: usize, limit_bytes: usize) -> IonInspector<'input> {
-        let mut reader = Reader::new(BinaryIonCursor::new(io::Cursor::new(input)));
-        reader.set_symtab_event_handler(SystemLevelEventSummarizer::new(out.clone()));
+impl<'a> IonInspector<'a> {
+    fn new<'b>(
+        input: &'b [u8],
+        out: &'b mut OutputRef,
+        bytes_to_skip: usize,
+        limit_bytes: usize,
+    ) -> IonInspector<'b> {
+        let reader = SystemReader::new(RawBinaryReader::new(io::Cursor::new(input)));
         let text_ion_writer = TextWriter::new(Vec::with_capacity(TEXT_WRITER_INITIAL_BUFFER_SIZE));
         IonInspector {
             output: out,
@@ -344,7 +281,35 @@ impl<'input> IonInspector<'input> {
         // appear each time some number of values is skipped.
         let mut bytes_skipped_this_level = 0;
 
-        while let Some((ion_type, _is_null)) = self.reader.next()? {
+        loop {
+            let (ion_type, _is_null) = match self.reader.next()? {
+                None => break,
+                Some(SystemStreamItem::VersionMarker(major, minor)) => {
+                    if major != 1 || minor != 0 {
+                        bail!(
+                            "Only Ion 1.0 is supported. Found IVM for v{}.{}",
+                            major,
+                            minor
+                        );
+                    }
+                    output(
+                        self.output,
+                        None,
+                        Some(4),
+                        SYSTEM_EVENT_INDENTATION,
+                        IVM_HEX,
+                        IVM_TEXT.dimmed(),
+                    )
+                    .expect("output() failure from on_ivm()");
+                    continue;
+                }
+                Some(SystemStreamItem::SymbolTableData(ion_type, is_null))
+                | Some(SystemStreamItem::Value(ion_type, is_null)) => {
+                    // We don't care if this is a system or user-level value; that distinction
+                    // is handled inside the SystemReader.
+                    (ion_type, is_null)
+                }
+            };
             // See if we've already processed `bytes_to_skip` bytes; if not, move to the next value.
             let complete_value_range = self.complete_value_range();
             if complete_value_range.end <= self.bytes_to_skip {
@@ -353,7 +318,9 @@ impl<'input> IonInspector<'input> {
             }
 
             // Saturating subtraction: if the result would underflow, the answer will be zero.
-            let bytes_processed = complete_value_range.start.saturating_sub(self.bytes_to_skip);
+            let bytes_processed = complete_value_range
+                .start
+                .saturating_sub(self.bytes_to_skip);
             // See if we've already processed `limit_bytes`; if so, stop processing.
             if bytes_processed >= self.limit_bytes {
                 let limit_message = if self.reader.depth() > 0 {
@@ -362,7 +329,7 @@ impl<'input> IonInspector<'input> {
                     "// --limit-bytes reached, ending."
                 };
                 output(
-                    &self.output,
+                    self.output,
                     None,
                     None,
                     &self.indentation_buffer,
@@ -377,9 +344,13 @@ impl<'input> IonInspector<'input> {
             // to get to this point, make a note of it in the output.
             if bytes_skipped_this_level > 0 {
                 self.text_buffer.clear();
-                write!(&mut self.text_buffer, "// Skipped {} bytes of user-level data", bytes_skipped_this_level)?;
+                write!(
+                    &mut self.text_buffer,
+                    "// Skipped {} bytes of user-level data",
+                    bytes_skipped_this_level
+                )?;
                 output(
-                    &self.output,
+                    self.output,
                     None,
                     None,
                     &self.indentation_buffer,
@@ -401,8 +372,11 @@ impl<'input> IonInspector<'input> {
                     self.inspect_level()?;
                     self.reader.step_out()?;
                     // Print the container's closing delimiter: }, ), or ]
+                    // FIXME: This should also print a trailing `,` if the parent context is
+                    //        a list or struct. See this issue for details:
+                    //        https://github.com/amzn/ion-cli/issues/17
                     output(
-                        &self.output,
+                        self.output,
                         None,
                         None,
                         &self.indentation_buffer,
@@ -419,13 +393,12 @@ impl<'input> IonInspector<'input> {
     }
 
     fn increase_indentation(&mut self) {
-        // Remove a level's worth of indentation from the buffer.
+        // Add a level's worth of indentation to the buffer.
         if self.reader.depth() > 0 {
             self.indentation_buffer.push_str(LEVEL_INDENTATION);
         }
     }
 
-
     fn decrease_indentation(&mut self) {
         // Remove a level's worth of indentation from the buffer.
         if self.reader.depth() > 0 {
@@ -435,11 +408,18 @@ impl<'input> IonInspector<'input> {
     }
 
     fn write_field_if_present(&mut self) -> IonResult<()> {
-        if let Some(field_id) = self.reader.field_id() {
+        if let Some(field_token) = self.reader.raw_field_name_token() {
+            let field_id = field_token.local_sid().expect("No SID for field name.");
             self.hex_buffer.clear();
-            to_hex(&mut self.hex_buffer, self.reader.raw_field_id_bytes().unwrap());
+            to_hex(
+                &mut self.hex_buffer,
+                self.reader.raw_field_id_bytes().unwrap(),
+            );
 
-            let field_name = self.reader.field_name().expect("Field ID present, name missing.");
+            let field_name = self
+                .reader
+                .field_name()
+                .expect("Field ID present, name missing.");
             self.text_buffer.clear();
             write!(&mut self.text_buffer, "'{}':", field_name)?;
 
@@ -447,7 +427,7 @@ impl<'input> IonInspector<'input> {
             write!(&mut self.color_buffer, " // ${}:", field_id)?;
             write!(&mut self.text_buffer, "{}", &self.color_buffer.dimmed())?;
             output(
-                &self.output,
+                self.output,
                 self.reader.field_id_offset(),
                 self.reader.field_id_length(),
                 &self.indentation_buffer,
@@ -459,24 +439,37 @@ impl<'input> IonInspector<'input> {
     }
 
     fn write_annotations_if_present(&mut self) -> IonResult<()> {
-        let num_annotations = self.reader.annotation_ids().len();
+        let num_annotations = self.reader.raw_annotations().count();
         if num_annotations > 0 {
             self.hex_buffer.clear();
-            to_hex(&mut self.hex_buffer, self.reader.raw_annotations_bytes().unwrap());
+            to_hex(
+                &mut self.hex_buffer,
+                self.reader.raw_annotations_bytes().unwrap(),
+            );
 
             self.text_buffer.clear();
             write!(&mut self.text_buffer, "'")?;
-            join_into(&mut self.text_buffer, "'::'", self.reader.annotations());
+            join_into(
+                &mut self.text_buffer,
+                "'::'",
+                self.reader.annotations().map(|o| o.unwrap()),
+            );
             write!(&mut self.text_buffer, "'::")?;
 
             self.color_buffer.clear();
             write!(&mut self.color_buffer, " // $")?;
-            join_into(&mut self.color_buffer, "::$", self.reader.annotation_ids().iter());
+            join_into(
+                &mut self.color_buffer,
+                "::$",
+                self.reader
+                    .raw_annotations()
+                    .map(|a| a.local_sid().unwrap()),
+            );
             write!(&mut self.color_buffer, "::")?;
 
             write!(self.text_buffer, "{}", self.color_buffer.dimmed())?;
             output(
-                &self.output,
+                self.output,
                 self.reader.annotations_offset(),
                 self.reader.annotations_length(),
                 &self.indentation_buffer,
@@ -495,7 +488,10 @@ impl<'input> IonInspector<'input> {
         self.format_value()?;
 
         self.hex_buffer.clear();
-        to_hex(&mut self.hex_buffer, self.reader.raw_header_bytes().unwrap());
+        to_hex(
+            &mut self.hex_buffer,
+            self.reader.raw_header_bytes().unwrap(),
+        );
         // Only write the bytes representing the body of the value if it is a scalar.
         // If it is a container, `inspect_level` will handle stepping into it and writing any
         // nested values.
@@ -505,9 +501,10 @@ impl<'input> IonInspector<'input> {
         }
 
         const TYPE_DESCRIPTOR_SIZE: usize = 1;
-        let length = TYPE_DESCRIPTOR_SIZE + self.reader.header_length() + self.reader.value_length();
+        let length =
+            TYPE_DESCRIPTOR_SIZE + self.reader.header_length() + self.reader.value_length();
         output(
-            &self.output,
+            self.output,
             Some(self.reader.header_offset()),
             Some(length),
             &self.indentation_buffer,
@@ -535,7 +532,9 @@ impl<'input> IonInspector<'input> {
         comment_buffer.clear();
 
         let writer = text_ion_writer; // Local alias for brevity.
-        let ion_type = reader.ion_type().expect("format_value() called when reader was exhausted");
+        let ion_type = reader
+            .ion_type()
+            .expect("format_value() called when reader was exhausted");
         if reader.is_null() {
             writer.write_null(reader.ion_type().unwrap())?;
         } else {
@@ -545,10 +544,11 @@ impl<'input> IonInspector<'input> {
                 Integer => writer.write_i64(reader.read_i64()?.unwrap()),
                 Float => writer.write_f64(reader.read_f64()?.unwrap()),
                 Decimal => writer.write_big_decimal(&reader.read_big_decimal()?.unwrap()),
-                Timestamp => writer.write_datetime(&reader.read_datetime()?.unwrap()),
+                Timestamp => writer.write_timestamp(&reader.read_timestamp()?.unwrap()),
                 Symbol => {
                     // TODO: Make this easier in the reader
-                    let sid = reader.read_symbol_id()?.unwrap();
+                    let symbol_token = reader.read_raw_symbol()?.unwrap();
+                    let sid = symbol_token.local_sid().unwrap();
                     let text = reader
                         .symbol_table()
                         .text_for(sid)
@@ -597,32 +597,43 @@ const CHARS_PER_HEX_BYTE: usize = 3;
 const HEX_BYTES_PER_ROW: usize = 8;
 const HEX_COLUMN_SIZE: usize = HEX_BYTES_PER_ROW * CHARS_PER_HEX_BYTE;
 
-fn write_header(output: &OutputRef) -> IonResult<()> {
-    // Unwrap our Rc<RefCell<dyn Write>> to get a &mut dyn Write for the rest of the function
-    let mut output = output.borrow_mut();
-
+fn write_header(output: &mut OutputRef) -> IonResult<()> {
     let line = "-".repeat(24 + 24 + 9 + 9 + (COLUMN_DELIMITER.len() * 3));
 
     writeln!(output, "{}", line)?;
-    write!(output, "{:^9}{}", "Offset".bold().bright_white(), COLUMN_DELIMITER)?;
-    write!(output, "{:^9}{}", "Length".bold().bright_white(), COLUMN_DELIMITER)?;
-    write!(output, "{:^24}{}", "Binary Ion".bold().bright_white(), COLUMN_DELIMITER)?;
+    write!(
+        output,
+        "{:^9}{}",
+        "Offset".bold().bright_white(),
+        COLUMN_DELIMITER
+    )?;
+    write!(
+        output,
+        "{:^9}{}",
+        "Length".bold().bright_white(),
+        COLUMN_DELIMITER
+    )?;
+    write!(
+        output,
+        "{:^24}{}",
+        "Binary Ion".bold().bright_white(),
+        COLUMN_DELIMITER
+    )?;
     writeln!(output, "{:^24}", "Text Ion".bold().bright_white())?;
     writeln!(output, "{}", line)?;
     Ok(())
 }
 
 // Accepting a `T` allows us to pass in `&str`, `&String`, `&ColoredString`, etc as out text_column
-fn output<T: Display>(output: &OutputRef,
-                      offset: Option<usize>,
-                      length: Option<usize>,
-                      indentation: &str,
-                      hex_column: &str,
-                      text_column: T) -> IonResult<()> {
-
-    // Unwrap our Rc<RefCell<dyn Write>> to get a &mut dyn Write for the rest of the function
-    let mut output = output.borrow_mut();
-
+// TODO: This could be a method on IonInspector
+fn output<T: Display>(
+    output: &mut OutputRef,
+    offset: Option<usize>,
+    length: Option<usize>,
+    indentation: &str,
+    hex_column: &str,
+    text_column: T,
+) -> IonResult<()> {
     // The current implementation always writes a single line of output for the offset, length,
     // and text columns. Only the hex column can span multiple rows.
     // TODO: It would be nice to allow important hex bytes (e.g. type descriptors or lengths)
@@ -689,7 +700,7 @@ fn closing_delimiter_for(container_type: IonType) -> &'static str {
         IonType::List => "]",
         IonType::SExpression => ")",
         IonType::Struct => "}",
-        _ => panic!("Attempted to close non-container type {:?}", container_type)
+        _ => panic!("Attempted to close non-container type {:?}", container_type),
     }
 }
 
@@ -703,8 +714,11 @@ fn to_hex(buffer: &mut String, bytes: &[u8]) {
     }
 }
 
-fn join_into<T: Display>(buffer: &mut String,
-                         delimiter: &str, mut values: impl Iterator<Item=T>) {
+fn join_into<T: Display>(
+    buffer: &mut String,
+    delimiter: &str,
+    mut values: impl Iterator<Item = T>,
+) {
     if let Some(first) = values.next() {
         write!(buffer, "{}", first).unwrap();
     }
@@ -712,4 +726,3 @@ fn join_into<T: Display>(buffer: &mut String,
         write!(buffer, "{}{}", delimiter, value).unwrap();
     }
 }
-
